const styleTableId = "#styleTable";
const styleModalId = "#styleFormModal";


// region Init Module

function initMessageStylesModule() {
    initializeDataTable(
        styleTableId,
        [
            {
                title: "Name",
                data: "name",
                render: function(data) {
                    const name = sanitise(data);
                    return `<button type="button" class="btn btn-link text-start text-decoration-none edit-modal">${name}</button>`;
                }
            },
            {
                title: "Is Embed",
                data: "is_embed",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Is Hyperlinked",
                data: "is_hyperlinked",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Show Author",
                data: "show_author",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Show Timestamp",
                data: "show_timestamp",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Show Images",
                data: "show_images",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Fetch Images",
                data: "fetch_images",
                className: "text-center",
                render: function(data) {
                    const iconClass = data ? "bi-check-circle-fill text-success" : "bi-x-circle-fill text-danger";
                    return `<i class="bi ${iconClass}"></i>`;
                }
            },
            {
                title: "Title Mutator",
                data: "title_mutator"
            },
            {
                title: "Description Mutator",
                data: "description_mutator"
            }
        ]
    );
};


// region Load Data

$(document).on("selectedServerChange", async function() {
    await loadMessageStyleData();
});

$(document).on("doDataLoad", async function() {
    await loadMessageStyleData();
});

async function loadMessageStyleData() {
    if (!selectedServer) {
        return;
    }

    setTableFilter(styleTableId, "server", selectedServer.id);
    await loadTableData(styleTableId, "/api/message-styles/", "GET");
}


// region New/Edit Modal

$(styleTableId).closest(".js-tableBody").siblings(".js-tableFilters").on("click", ".table-new-btn", async function() {
    await openStyleModal(-1);
});

$(styleTableId).on("click", ".edit-modal", async function() {
    let id = $(styleTableId).DataTable().row($(this).closest("tr")).data().id;
    await openStyleModal(id);
});

async function openStyleModal(id) {
    $modal = $(styleModalId);

    if (parseInt(id) === -1) {
        $modal.find(".form-create").show();
        $modal.find(".form-edit").hide();

        $modal.find("[data-field]").each(function() {
            const type = $(this).attr("type");
            const defaultVal = $(this).attr("data-default") || "";

            if (type === "checkbox") {
                $(this).prop("checked", defaultVal === "true");
            }
            else if (type === "datetime-local") {
                $(this).val(getCurrentDateTime());
            }
            else {
                $(this).val(defaultVal).change();
            }
        });
    }
    else {
        $modal.find(".form-create").hide();
        $modal.find(".form-edit").show();

        const data = await ajaxRequest(`/api/message-styles/${id}/`, "GET");

        $modal.find("[data-field]").each(function() {
            const key = $(this).attr("data-field");
            const value = data[key];

            if (typeof value === "boolean") {
                $(this).prop("checked", value);
            }
            else if (isISODateTimeString(value)) {
                $(this).val(value.split('+')[0].substring(0, 16));
            }
            else {
                $(this).val(value).change();
            }
        });
    }

    $modal.modal("show");
}


// region Load Mutator Options

$(document).ready(async function() {
    await loadMutatorOptions();
});

async function loadMutatorOptions() {
    let $inputs = $(styleModalId).find('[data-field="title_mutator"], [data-field="description_mutator"]');

    $inputs.val("").change();
    $inputs.prop("disabled", true);

    $inputs.find("option").each(function() {
        if ($(this).val()) {
            $(this).remove();
        }
    });

    const data = await ajaxRequest("/api/message-mutators/?page_size=25", "GET");
    data.results.forEach(mutator => {
        $inputs.append($(
            "<option>",
            {text: mutator.name, value: mutator.id}
        ));
    });

    $inputs.prop("disabled", false);
}
